#!/usr/bin/perl

# Name: ricocheting's Error-Lumberjack (logger) with administration options
# Author: ricocheting
# http://www.ricocheting.com/perl/
# Version: 2.5
# Date: 02-03-02
#
# This script is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
# 

# Path to log file (where all the errors will be stored)
$config{'log'}= 'log.txt';

# Email address (used on error messages as a contact)
$config{'email'}= 'myname@domain.com';

# Homepage address (for default "back to hompage" link)
$config{'homepage'}= 'http://yourdomain.com';


# Maximum number of errors to log before oldest ones start getting deleted
# (so if you forget and let it sit for 6 months, you don't come to find a 20mb log file)
$config{'maxEntries'}=5000;

# if no custom page for the error is selected, this script handles it
# $error{'500'}="http://www.yourdomain.com/path/myErrorPage500.html";
$error{'403'}="";
$error{'404'}="";
$error{'500'}="";


# If you don't set a custom page, you can configure your own 
# header which will be appended to the top of each default page.
$config{'header'} =<<"EOF";
Content-type: text/html

<html>
<head>
<title> Error $ENV{'QUERY_STRING'} </title>
<style type="text/css"><!--
td		{font-size: 10pt;}
body	{background-color:white; color:black; font-size: 10pt; font-family:VERDANA,HELVETICA,ARIAL;}
a:link	{text-decoration:underline;}
a:visited	{text-decoration:underline; color:#330066;}
a:hover	{text-decoration:none; color:#660066;}
//-->
</style>
</head><body>
<table border width="80%" align=center bordercolor="#cccccc" cellspacing=1><tr><td>

EOF
#-- Ends the custom header


# Footer for each HTML page generated
$config{'footer'}=<<"EOF";
<br><br>
</td></tr>

<tr><td align=center>
[<a href="javascript:history.back()">Back</a>] &nbsp; 
[<A HREF="$config{'homepage'}">Home</A>] &nbsp; 
[<A HREF="mailto:$config{'email'}">E-mail</A>]
</td></tr>
</table>
</body></html>
EOF

# <----------- to change the default messages displayed, jump down to the bottom ----------->

###############
# constructor #


&logCurrent(); #adds the current error to the log


if(length($error{$ENV{'QUERY_STRING'}}) > 0){#if they created a special error, go there
	print "Location: $error{$ENV{'QUERY_STRING'}}\n\n";}

elsif($ENV{'QUERY_STRING'} eq '403'){#default "restricted"
	&explain_403;}
elsif($ENV{'QUERY_STRING'} eq '404'){#default "not found"
	&explain_404;}
elsif($ENV{'QUERY_STRING'} eq '500'){#default "server error"
	&explain_500;}
else{#general error
	&explain_default;}

#-# end constructor
###############


#-#############################################
# Sub: logCurrent
# desc: logs the current error to file
sub logCurrent{
open(FILE, "<$config{'log'}");
my @errors=<FILE>;
close(FILE);

if($#errors >= $config{'maxEntries'}){
	&shortenLog();
}

open(FILE, ">>$config{'log'}");
print FILE time.'||'.$ENV{'REMOTE_ADDR'}.'||'.$ENV{'QUERY_STRING'}.'||'.$ENV{'REDIRECT_URL'}.'||'.$ENV{'HTTP_REFERER'}."\n";
close(FILE);
}#-#end sub logCurrent()


#-#############################################
# Sub: logCurrent
# desc: shortens the log by 10%
sub shortenLog{
open(FILE, "<$config{'log'}");
my @errors=<FILE>;
close(FILE);
chomp(@errors);

#start at 90% of long
my $start= sprintf("%.0f", ($config{'maxEntries'} - ($config{'maxEntries'}*.8)));

open(FILE, ">$config{'log'}");
	for (my $i=$start; $i<=$#errors;$i++){
	print FILE "$errors[$i]\n";
	}
close(FILE);
}#-#end sub shortenLog()


#-#############################################
# Sub: explain_403
# desc: print out error 403 (access restricted)
sub explain_403{

print <<"EOF";
$config{'header'}

<h1>Restricted Access</h1>
The document you requested has restricted access and you do not 
have permission to view it. If you have misplaced your password, or 
have a compelling reason to view the restricted information, please 
feel free to <A HREF="mailto:$config{'email'}">contact us</A>.
$config{'footer'}

EOF
}#-#end sub explain_403


#-#############################################
# Sub: explain_404
# desc: print out error 404 (file not found)
sub explain_404{

print <<"EOF";
$config{'header'}

<h1>File Not Found</h1>
The document you were searching for was not found.  It has 
either been moved, never existed, or might have just been a 
mistyped. To find documents which have moved, you might try 
starting at the <A HREF="$config{'homepage'}">main page</A>. 
Also, if you feel this page should exist, send us a 
<a href="mailto:$config{'email'}">email</A> so that we can 
correct this problem.
$config{'footer'}

EOF
}#-#end sub explain_404


#-#############################################
# Sub: explain_500
# desc: print out error 500 (server error)
sub explain_500{

print <<"EOF";
$config{'header'}

<h1>Server Error</h1>
The script you are trying to access has suffered a fatal error. 
Please try again at a later time or feel free to 
<a href="mailto:$config{'email'}">email us</a> and make sure we 
are aware of the problem and can correct it. Include any 
information that might be helpful in solving this problem (such 
as time the error occurred, and anything you might have done 
to recieve this error).
$config{'footer'}

EOF
}#-#end sub explain_500


#-#############################################
# Sub: explain_default
# desc: print out default message if error is unknown
sub explain_default{

print <<"EOF";
$config{'header'}

<h1>Error</h1>
There was an unexplained error in retrieving your requested file. You
may wish to try again from the <A HREF="$config{'homepage'}">main page</A>, or
<a href="mailto:$config{'email'}">email us</a> with any
additional details about how you reached this error message.
$config{'footer'}

EOF
}#-#end sub explain_default
